# -*- coding: utf-8 -*-
"""
Created on Mon Sep 23 13:05:34 2019

@author: tadahaya
"""
import unittest
import pandas as pd
import os
import sys
import math

from enan.connect import Connect

class SampleTest(unittest.TestCase):
    CLS_VAL = 'none'

    # called when test class initialization
    @classmethod
    def setUpClass(cls):
        if sys.flags.debug:
            print('> setUpClass method is called.')
        cls.CLS_VAL = '> setUpClass : initialized!'
        if sys.flags.debug:
            print(cls.CLS_VAL)

    # called when test class end
    @classmethod
    def tearDownClass(cls):
        if sys.flags.debug:
            print('> tearDownClass method is called.')
        cls.CLS_VAL = '> tearDownClass : released!'
        if sys.flags.debug:
            print(cls.CLS_VAL)

    # called when a test method runs
    def setUp(self):
        if sys.flags.debug:
            print(os.linesep + '> setUp method is called.')
        self.smpl = Connect()

    # called when a test method ends
    def tearDown(self):
        if sys.flags.debug:
            print(os.linesep + '> tearDown method is called.')

    def _df_checker(self,df):
        if type(df)!=pd.core.frame.DataFrame:
            return False
        elif df.shape[0]==0:
            return False
        else:
            head = df.head(1)
            judge = math.isnan(head.iat[0,0])
            return not judge

    def _sr_checker(self,sr):
        if type(sr)!=pd.core.series.Series:
            return False
        if sr.shape[0]==0:
            return False
        else:
            head = sr.head(1)
            judge = math.isnan(head.iat[0])
            return not judge

    def test_calc(self):
        ### preparation
        ref,obj = self.smpl.generate_test_data()
        self.smpl.fit(ref)

        ### test
        self.assertTrue(self._df_checker(self.smpl.calc(data=obj)))


# -*- coding: utf-8 -*-
"""
Created on Mon Sep 23 13:05:34 2019

ssGSEA class

@author: tadahaya
"""
import pandas as pd
import numpy as np
from itertools import chain
import random
import string
from tqdm import tqdm

from .process.processor import Processor
from .analyzer import Analyzer
from .data.data_control import ssGSEADataControl
from .calculator._gsea import Calculator
from .plot._plot import PlotSsGSEA

# concrete class
class ssGSEA(Analyzer):
    def __init__(self):
        self.data = ssGSEADataControl()
        self.__process = Processor()
        self.__calc = Calculator()
        self.__plot = PlotSsGSEA()
        self.__whole = set()
        self.__ref = dict()
        self.__obj = pd.DataFrame()
        self.__method = ""
        self.alpha = 0.0
        self.__fterm = None
        self.__mode = None
        self.res = pd.DataFrame()


    ### data processing ###
    def check_ref(self,keyword:str):
        """ check contents of reference data """
        if len(self.__ref)==0:
            raise ValueError("!! fit() before this process !!")
        try:
            temp = self.__ref[keyword]
            print("{0}: {1}".format(keyword,temp))
        except KeyError:
            print("!! Wrong keyword !!")
            hit = {v for v in self.__ref.keys() if keyword in v}
            print("perhaps: {}".format(hit))

    def vector2set(self,data,fold:float=3.0,
                   nmin:int=None,nmax:int=None,**kwargs):
        """
        convert dataframe to the outlier set for reference

        Parameters
        ----------
        data: dataframe
            feature x sample dataframe

        fold: float
            indicates fold change determining the outliers

        nmin,nmax: int
            indicate the minimum/maximum number of each set 

        """
        return self.__process.vec2set(mtx=data,fold=fold,nmin=nmin,nmax=nmax,
                                      two_sided=False,**kwargs)


    ### data control ###
    def fit(self,data:dict,keep_whole:bool=False,nmin=None):
        """
        set a reference data instance
        
        Parameters
        ----------
        data: dict
            a dictionary of sets like {"XXXX":{"aa","bb"},"YYYY":{"cc","dd","ee"},...}

        keep_whole: boolean
            whether whole features is conserved when already registered

        nmin: int
            indicates the number of features necessary for each set

        """
        self.data.set_ref(data=data)
        self.__ref = self.data.get_ref()
        if keep_whole:
            if len(self.__whole)==0:
                raise ValueError("!! set_whole() or turn off keep_whole !!")
            else:
                self.data.adjust_ref()
        else:
            self.__whole = set(chain.from_iterable(self.__ref.values()))
            self.data.set_whole(self.__whole)
        if nmin is not None:
            temp = self.__ref.copy()
            for k,v in self.__ref.items():
                if len(v) < nmin:
                    del temp[k]
            self.__ref = temp
            self.data.set_ref(data=self.__ref)
            
    def set_whole(self,whole:set):
        """
        set whole features
        
        Parameters
        ----------
        whole: set
            indicates whole features
        
        """
        self.data.set_whole(whole)
        self.__whole = self.data.get_whole()
        if len(self.data.get_ref())!=0:
            self.data.adjust_ref()

    def get_ref(self):
        """ get reference data instance """
        return self.data.get_ref()

    def get_obj(self):
        """ get an object data instance """
        return self.data.get_obj()

    def get_whole(self):
        return self.__whole


    def get_calculator(self):
        """ get calculater object """
        return self.__calc


    ### calculation ###
    def calc(self,data,fterm=None,method:str="standard",alpha:float=0.25): # realization
        """
        conduct ssGSEA

        Parameters
        -------
        data: dataframe
            feature x sample dataframe

        fterm: str or int
            indicate the term of interest or the corresponding No.

        method: str
            indicate a method for calculating the enrichment score
            "starndard": employed in the original paper Barbie, et al, 2009
            "kuiper": Kuiper test statistics, good when up/down genes are mixed, tail sensitive
            "gsva": GSVA like statistics, good when unidirection (ex. up only)

        alpha: float, (0,1]
            indicate weight of center
            0.25 is employed in the original paper (Barbie,et al.,2009)

        Returns res
        -------
        res: df
            gene set enrichment score

        """
        self.__method = method
        self.__alpha = alpha
        self.__fterm = fterm
        if method=="standard":
            self.__calc.to_standard()
            print("Standard method",flush=True)
        elif method=="kuiper":
            self.__calc.to_kuiper()
            print("Kuiper method",flush=True)
        elif method=="gsva":
            self.__calc.to_gsva()
            print("GSVA method",flush=True)
        else:
            raise ValueError("!! Wrong method: choose 'standard', 'kuiper', or 'gsva' !!")
        self.data.set_obj(data)
        self.__obj = self.data.get_obj()
        temp = self.__obj.copy()
        col = list(temp.columns)
        if fterm is None:
            self.__mode = "exploratory"
            self.__calc.to_expssgsea()
            res = []
            ap = res.append
            for v in tqdm(col):
                ap(self.__calc.calc(obj=temp[v],ref=self.__ref,alpha=alpha))
            res = pd.concat(res,axis=1,join="inner")
            res.columns = col
        else:
            self.__mode = "focused"
            self.__calc.to_ssgsea()
            self.__fterm = fterm
            res = self.__calc.calc(obj=temp,ref=self.__ref,alpha=alpha,fterm=fterm)
        self.res = res
        return res


    def normalize_score(self):
        """
        normalize enrichment score with maximum value
        Note that this method can be applied
        only when the standard method was employed for calculation
        
        """
        if self.__mode=="focused":
            raise ValueError("!! Only applicable to the exploratory mode !!")
        n = len(self.__whole)
        if self.res.empty:
            raise ValueError("!! No result stored: calc() before this method !!")
        key = list(self.__ref.keys())
        val = list(self.__ref.values())
        dic = dict(zip(key,[len(v) for v in val]))
        idx = list(self.res.index)
        t = np.array([dic[v] for v in idx])
        if type(self.res)==type(pd.DataFrame()):
            kmax = np.c_[1 - t/n]
            val = self.res.values/kmax
        else:
            kmax = 1 - t/n
            val = self.res.values.flatten()/kmax
        return pd.DataFrame(val,columns=self.res.columns,index=idx)


    ### visualization ###
    def set_res(self,data):
        """ set a result """
        self.res = data


    def plot(self,keyword:list=[],fterm:str=None,mode:str=None,**kwargs): # realization
        """
        visualize a result of enrichment analysis

        Parameters
        ----------
        fterm: str
            indicate the group of interest

        mode: str
            indicate ssGSEA mode: 'exploratory' or 'focused'
            if None (default), plot the current result

        keyword: list
            indicate samples to be visualized

        fileout: str
            indicate the path for the output image

        dpi: int
            indicate dpi of the output image
            
        xlabel,ylabel: str
            indicate the name of x and y axes

        title: str
            indicate the title of the plot

        color: str
            indicate the color of swarmplot (if sample size is less than 30)

        palette: list
            indicate color of boxplot

        alpha: float
            indicate transparency of the bars: (0,1)

        size: float
            size of the markers

        fontsize: float
            indicate the fontsize in the plot

        textsize: float
            indicate the fontsize of the texts in the bars

        figsize: tuple
            indicate the size of the plot

        """
        if mode is None: # plot the current result
            if self.__mode=="exploratory":
                # exploratory mode
                if fterm is None:
                    raise ValueError("!! Indicate fterm (focused term) !!")
                else:
                    data = self.res.T[fterm]
                    self.__plot.plot(data=data,keyword=keyword,focus=fterm,**kwargs)
            elif self.__mode=="focused":
                # focused mode
                self.__plot.plot(data=self.res,keyword=keyword,focus=self.__fterm,**kwargs)
            else:
                raise ValueError("!! Indicate mode: 'exploratory' or 'focused' !!")
        elif mode=="exploratory": # plot a loaded result
            # exploratory mode
            if fterm is None:
                raise ValueError("!! Indicate fterm (focused term) !!")
            else:
                data = self.res.T[fterm]
                self.__plot.plot(data=data,keyword=keyword,focus=fterm,**kwargs)
        elif mode=="focused": # plot a loaded result
            # focused mode
            if fterm is None:
                raise ValueError("!! Indicate fterm (focused term) !!")
            else:
                self.__plot.plot(data=self.res,keyword=keyword,focus=fterm,**kwargs)
        else:
            raise ValueError("!! Wrong mode: 'exploratory' or 'focused' !!")


    ### other ###
    def generate_test_data(self):
        """ generate data for test """    
        # prepare reference
        key = [''.join(random.choices(string.ascii_letters, k=5)) for i in range(10)]

        def fxn():
            return {random.randint(0,50) for j in range(random.randint(10,100))}

        val = [fxn() for i in range(9)] + [{1,2,3,4,5,6,7}]
        ref = dict(zip(key,val))

        # prepare object
        col = ["control_{}".format(i + 1) for i in range(5)]
        col += ["treated_{}".format(i + 1) for i in range(5)]
        obj = pd.DataFrame([np.random.randn(10000) for i in range(10)],index=col).T
        print("generate test data as follows:")
        print('    ref: dict, {"XXXX":{"aa","bb"},"YYYY":{"cc","dd","ee"},...}')
        print('    obj: dataframe, feature x sample')
        return ref,obj


# -*- coding: utf-8 -*-
"""
Created on Mon Sep 23 13:05:34 2019

Connect class

@author: tadahaya
"""
import pandas as pd
import numpy as np
from itertools import chain
import copy
import random
import string

from .process.processor import Processor
from .analyzer import Analyzer
from .data.data_control import ConnectivityDataControl
from .calculator._connectivity import Calculator
from .plot._plot import PlotGSEA

# concrete class
class Connect(Analyzer):
    def __init__(self):
        self.data = ConnectivityDataControl()
        self.__process = Processor()
        self.__calc = Calculator()
        self.__plot = PlotGSEA()
        self.__whole = set()
        self.__ref = dict()
        self.__obj = set()
        self.res = pd.DataFrame()


    ### data processing ###
    def check_ref(self,keyword:str):
        """ check contents of reference data """
        if len(self.__ref)==0:
            raise ValueError("!! fit() before this process !!")
        try:
            temp = self.__ref[keyword]
            print("up: {}".format(temp[0]))
            print("")
            print("down: {}".format(temp[0]))
        except KeyError:
            print("!! Wrong keyword !!")
            hit = {v for v in self.__ref.keys() if keyword in v}
            print("perhaps: {}".format(hit))

    def vector2set(self,data,fold:float=3.0,
                   nmin:int=None,nmax:int=None,**kwargs):
        """
        convert dataframe to the outlier set for reference data

        Parameters
        ----------
        data: dataframe
            feature x sample dataframe

        fold: float
            indicates fold change determining the outliers

        nmin,nmax: int
            indicate the minimum/maximum number of each set 

        """
        return self.__process.vec2set(mtx=data,fold=fold,nmin=nmin,nmax=nmax,
                                      two_sided=True,**kwargs)


    ### data control ###
    def fit(self,data:dict,keep_whole:bool=False,nmin=None):
        """
        set a reference data instance
        
        Parameters
        ----------
        data: dict of up-/down-tags
            keys: tag name
            values: tuple of up-/down-gene set
            {tag_name:(up-tag set,down-tag set)}

        keep_whole: boolean
            whether whole features is conserved when already registered

        nmin: int
            indicates the number of features necessary for each set

        """
        self.data.set_ref(data=data)
        self.__ref = self.data.get_ref()
        if keep_whole:
            if len(self.__whole)==0:
                raise ValueError("!! set_whole() or turn off keep_whole !!")
            else:
                self.data.adjust_ref()
        else:
            temp = [v[0]|v[1] for v in self.__ref.values()]
            self.__whole = set(chain.from_iterable(temp))
            self.data.set_whole(self.__whole)
        if nmin is not None:
            temp = self.__ref.copy()
            for k,v in self.__ref.items():
                if (len(v[0]) < nmin) or (len(v[1]) < nmin):
                    del temp[k]
            self.__ref = temp
            self.data.set_ref(data=self.__ref)

    def set_whole(self,whole:set):
        """
        set whole features
        
        Parameters
        ----------
        whole: set
            indicates whole features
        
        """
        self.data.set_whole(whole)
        self.__whole = self.data.get_whole()
        if len(self.data.get_ref())!=0:
            self.data.adjust_ref()

    def get_ref(self):
        """ get reference data instance """
        return self.data.get_ref()

    def get_obj(self):
        """ get an object data instance """
        return self.data.get_obj()

    def get_whole(self):
        return self.__whole


    ### calculation ###
    def calc(self,data): # realization
        """
        conduct connectivity analysis

        Parameters
        ----------
        data: dataframe
            feature x sample dataframe

        Returns res
        -------
        res: df
            gene set enrichment score

        """
        self.data.set_obj(data)
        self.__obj = self.data.get_obj()
        temp = self.__obj.copy()
        ref_ins = copy.deepcopy(self.data.ref)
        ref_ins.set_whole(set(temp.index))
        ref_ins.adjust()
        self.res = self.__calc.calc(obj=temp,ref=ref_ins.get_data())
        del ref_ins
        return self.res


    ### visualization ###
    def set_res(self,data):
        """ set a result """
        self.res = data


    def plot(self,sample_name:str=None,highlight:list=[],ylabel:str="connectivity score",**kwargs): # realization
        """
        visualize a result of connectivity score

        Parameters
        ----------
        sample_name: str
            indicate the sample name to be visualized

        highlight: list
            indicate the plots to be highlightened

        fileout: str
            indicate the path for the output image

        dpi: int
            indicate dpi of the output image
            
        ylabel: str
            indicate the name of y axis

        title: str
            indicate the title of the plot

        color: str
            indicate the color of the bars

        fontsize: float
            indicate the fontsize in the plot

        size: float
            indicate the size of the plot

        figsize: tuple
            indicate the size of the plot

        """
        if sample_name is None:
            col = list(self.res.columns)
            for v in col:
                self.__plot.plot(data=self.res[v],highlight=highlight,ylabel=ylabel,**kwargs)
        else:
            focused = self.res[sample_name]
            self.__plot.plot(data=focused,highlight=highlight,ylabel=ylabel,**kwargs)


    ### other ###
    def generate_test_data(self):
        """ generate data for test """    
        # prepare reference
        key = [''.join(random.choices(string.ascii_letters, k=5)) for i in range(10)]
        ref = pd.DataFrame([np.random.randn(10000) for i in range(10)],index=key).T
        temp = ref.iloc[:,0]
        ref = self.vector2set(ref)

        # prepare object
        col = ["xxxx","yyyy"]
        obj = pd.DataFrame([np.random.randn(10000) for i in range(2)],index=col).T
        obj["zzzz"] = temp
        print("generate test data as follows:")
        print('    ref: dict, {"XXXX":({"aa","bb"},{"cc","dd","ee"}),...}')
        print('    obj: dataframe, feature x sample')
        return ref,obj


# -*- coding: utf-8 -*-
"""
Created on Wed Apr  8 15:59:27 2020

algorithm of connectivity score calculation

@author: tadahaya
"""
import pandas as pd
import numpy as np
from scipy import stats
from scipy.stats import rankdata
from numpy import random as rnd

class Calculator():
    """ calculate connectivity score """
    def __init__(self):
        self.vj = []
        self.res = pd.DataFrame()


    def calc(self,obj,ref):
        """
        calculate connectivity scores

        Parameters
        ----------
        object : dataframe
            feature x sample dataframe
        
        ref: dict of up-/down-tags
            keys: tag name
            values: tuple of up-/down-gene set
            {tag_name:(up-tag set,down-tag set)}
    
        Returns
        -------
        connectivity score: float

        """
        self.vj = generate_v(obj)
        val = list(ref.values())
        score = []
        ap = score.append
        for t in val:
            temp = []
            ap2 = temp.append
            for v in self.vj:
                ap2(calc_kss(t,v))
            ap(temp)
        self.res = pd.DataFrame(score,columns=list(obj.columns),index=list(ref.keys()))
        return self.res


    def get_details(self):
        """
        obtain detailed data for visualization

        Returns (vj,res)
        ----------
        vj: dict
            dictionary indicating the locations of tag-positive features

        """
        return self.vj,self.res


############### functions #####################################################
def generate_v(target):
    """
    generate a lisf of dictionaries corresponding to V(j) in CMap; Lamb J, Science, 2006

    Parameters
    ----------
    target: dataframe
        target response data
        features x samples

    """
    target = pd.DataFrame(target.sort_index())
    ind = list(target.index)
    if len(target.columns) < 2:
        rank = rankdata(-target.T.values).tolist()
        dic = dict(zip(ind,rank))
        return [dic] # corresponding to V(j) in CMap
    else:
        ranks = [rankdata(-v).tolist() for v in target.T.values]
        dics = [dict(zip(ind,v)) for v in ranks] # list of Vj
        return dics    


def _kss_max(n,tu,td):
    """ to normalize the difference of KS max dependent on t """
    return 2 + 1/n - (tu + td)/n


def _ab(tag,vj):
    """
    calculate a and b in CMap; Lamb J, Science, 2006

    Parameters
    ----------
    tag: list
        a feature list for tag

    vj: dict
        dictionary corresponding to V(j) in CMap; Lamb J, Science, 2006
        key, gene; val, rank

    """
    t = len(tag)
    n = len(vj)
    rank = [vj[v] for v in tag]
    rank.sort()
    lst_a = []
    ap_a = lst_a.append
    lst_b = []
    ap_b = lst_b.append
    for j in range(t):
        a = (j + 1)/t - rank[j]/n
        b = rank[j]/n - j/t 
        ap_a(a)
        ap_b(b)
    a_max = np.max(lst_a)
    b_max = np.max(lst_b)
    return (a_max,b_max)


def calc_kss(tag,vj):
    """
    calculate Kolmogorov-Smirnov statistics as in CMap; Lamb J, Science, 2006

    Parameters
    ----------
    tag: tuple
        tuple of up-/down-gene lists; (up,down)
        sorted with the values in the descending order

    vj: dict
        dictionary corresponding to V(j) in CMap; Lamb J, Science, 2006
        key, gene; val, rank

    """
    a_up,b_up = _ab(tag[0],vj)
    a_dn,b_dn = _ab(tag[1],vj)
    if a_up > b_up:
        ks_up = a_up
    else:
        ks_up = -1*b_up
    if a_dn > b_dn:
        ks_dn = a_dn
    else:
        ks_dn = -1*b_dn
    if ks_up*ks_dn > 0:
        ks = 0
    else:
        ks = ks_up - ks_dn
    n = len(vj)
    tu = len(tag[0])
    td = len(tag[1])
    kssmax = _kss_max(n,tu,td)    
    return ks/kssmax            


# def calc_p(target,tag,kss,permutation=10000):
#     """
#     calculate p value of Kolmogorov-Smirnov statistics based on permutation
#     randomize target data to generate null distribution

#     Parameters
#     ----------
#     target: dataframe or series
#         target response data

#     tag: tuple
#         tuple of up-/down-gene lists; (up,down)
#         sorted with the values in the descending order
#         << Caution >> take a tag, not tags (because it is too time-consuming)

#     kss: default, None
#         KS score of real data

#     Return
#     ----------
#     float: permutation p value
#     dataframe: null distribution

#     """
#     target = pd.DataFrame(target.sort_index())
#     ind = list(target.index)
#     perm_set = int(permutation/1000)
#     if perm_set==0:
#         perm_set = 1
#     res = []
#     ap = res.append
#     print("permutation ({} times)".format(perm_set))
#     for i in range(perm_set):
#         print(i)
#         y = 2*stats.bernoulli.rvs(0.5,loc=0,size=len(ind)) - 1
#         rand = np.array([rnd.permutation(y) for i in range(1000)])
#         rand = pd.DataFrame(target.values.flatten()*rand,columns=target.index).T
#         vjs = generate_v(rand)
#         ap([calc_kss(tag,w) for w in vjs])
#     res = list(chain.from_iterable(res))
#     ks_p = np.count_nonzero(np.abs(np.array(res)) >= np.abs(kss))
#     if ks_p/permutation==1.0:
#         pval = 1.0
#     else:    
#         pval = 2*ks_p/permutation
#     return (pval,res)


# def calc_p_fast(tag,vj,kss,permutation=10000):
#     """
#     calculate p value of Kolmogorov-Smirnov statistics based on permutation
#     random sampling of up-/down-tags

#     Parameters
#     ----------
#     tag: tuple
#         tuple of up-/down-gene lists; (up,down)
#         sorted with the values in the descending order

#     vj: dict
#         dictionary corresponding to V(j) in CMap 2006,Science
#         key, gene; val, rank

#     Return
#     ----------
#     float: permutation p value
#     dataframe: null distribution

#     """
#     n_up = len(tag[0])
#     n_dn = len(tag[1])
#     keys = list(vj.keys())
#     res = []
#     ap = res.append
#     for i in range(permutation):
#         rnd.shuffle(keys)
#         tag_random = (keys[:n_up],keys[n_up:n_up + n_dn])
#         ap(calc_kss(tag_random,vj))
#     ks_p = np.count_nonzero(np.abs(np.array(res)) >= np.abs(kss))
#     pval = 2*ks_p/permutation
#     if pval > 1:
#         pval = 1.0
#     return (pval,res)


# -*- coding: utf-8 -*-
"""
Created on Mon Sep 23 13:05:34 2019

a module for plot

@author: tadahaya

"""
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
from matplotlib import gridspec

# abstract class
class Plot():
    def __init__(self):
        pass

    def plot(self):
        raise NotImplementedError        


# concrete class
class PlotFET(Plot):
    def plot(self,res,focus=5,fileout="",dpi=100,thresh=0.05,xlabel="-logP",ylabel="",title="",
                        color="royalblue",alpha=0.5,height=0.8,fontsize=12,textsize=12,figsize=()):
        """
        visualize a result of enrichment analysis

        Parameters
        ----------
        res: dataframe
            a result file of enrichment analysis
            
        """
        if len(res) < focus:
            focus = len(res)
        res = res.sort_values(by="p value")
        res = res.iloc[:focus,:]
        res2 = res[res["adjusted p value"] < thresh]
        val = -np.log10(res["adjusted p value"])
        val = [v for v in val[::-1]]
        val2 = -np.log10(res2["adjusted p value"])
        val2 = val2.tolist() + [np.nan]*(len(res) - len(res2))
        val2 = [v for v in val2[::-1]]
        name = list(res.index)
        name = [v.replace("_"," ") for v in name[::-1]]
        X = list(range(focus))
        if len(figsize) > 0:
            fig = plt.figure(figsize=figsize)
        else:
            fig = plt.figure()
        ax = fig.add_subplot(111)
        if len(xlabel) > 0:
            ax.set_xlabel(xlabel,fontsize=fontsize)
        if len(ylabel) > 0:
            ax.set_ylabel(ylabel,fontsize=fontsize)
        if len(title) > 0:
            ax.set_title(title,fontsize=fontsize)
        ax.tick_params(labelsize=fontsize)
        ax.set_yticks([])
        ax.spines["right"].set_color("none")
        ax.spines["bottom"].set_color("none")
        ax.spines["top"].set_color("none")
        ax.barh(X,val,color="lightgrey",alpha=alpha,height=height)
        ax.barh(X,val2,color=color,height=height,alpha=alpha)
        for v,w in zip(name,X):
            ax.text(0.02,w,v,ha="left",va="center",fontsize=textsize)
        if len(fileout) > 0:
            plt.savefig(fileout,bbox_inches="tight",dpi=dpi)
        plt.tight_layout()
        plt.show()


# concrete class
class PlotGSEA(Plot):
    def plot(self,data,highlight=[],color="goldenrod",label="",ylabel="",
             title="",fileout="",dpi=100,fontsize=14,size=40,figsize=()):
        """
        visualize a result of enrichment analysis
        
        Parameters
        ----------
        es: 2d array
            enrichment scores of each location (feature x tag/sample)
        focus: int
            indicate the No. of the column of interest
            
        """
        data = data.sort_values(ascending=False)
        idx = list(data.index)
        x = np.arange(1,len(data) + 1,1)
        y_f = data.loc[highlight]
        x_f = [idx.index(v) + 1 for v in list(y_f.index)]
        if figsize!=():
            g = plt.figure(figsize=figsize)
        else:
            g = plt.figure()
        ax = g.add_subplot(1,1,1)
        if len(title) > 0:
            ax.set_title(title,fontsize=fontsize)
        ax.scatter(x,data.values,color="lightgrey",s=size)
        ax.scatter(x_f,y_f,color=color,label=label,s=size)
        ax.set_ylabel(ylabel,fontsize=fontsize)
        if len(label) > 0:
            ax.legend(loc="best",fontsize=fontsize)
        if len(fileout) > 0:
            plt.savefig(fileout,bbox_inches="tight",dpi=dpi)
        ax.tick_params(labelsize=fontsize)
        plt.tight_layout()
        plt.show()


    def plot_running(self,es,loc,focus=None,fileout="",dpi=100,xlabel="",ylabel="",title="",
                        color="forestgreen",alpha:float=0.2,fontsize=12,figsize=(),
                        barcode_params:dict=None):
        """
        visualize a result of enrichment analysis
        
        Parameters
        ----------
        es: 2d array
            enrichment scores of each location (feature x tag/sample)

        loc: 2d array
            location of the indicated tag members in the specimen data

        focus: int
            indicate the No. of the column of interest
            
        """
        BARCODE = {'color':'indigo','size':0.15} # hard coding
        if barcode_params is not None:
            for k,v in barcode_params.items():
                BARCODE[k] = v
        y = es.T[focus]
        x = np.arange(1,len(y) + 1,1)
        xmin = np.min(x)
        xmax = np.max(x)
        if figsize!=():
            fig = plt.figure(figsize=figsize)
        else:
            fig = plt.figure()
        spec = gridspec.GridSpec(ncols=1,nrows=2,
                                height_ratios=[1,BARCODE['size']])
        # prep axes
        if len(title) > 0:
            plt.suptitle(title)
        plt.rcParams['font.size'] = fontsize
        ax = fig.add_subplot(spec[0,0])
        axb = fig.add_subplot(spec[1,0],sharex=ax)
        # plot running
        ax = del_spines(ax,['top','right','bottom'])
        ax.plot(x,y,color=color)
        ax.fill_between(x,0,y,facecolor=color,alpha=alpha)
        ax.set_xlim(xmin,xmax)
        ax.xaxis.set_visible(False)
        # plot barcode
        axb = del_spines(axb,['top','right','bottom','left'])
        locf = loc[focus]
        yscale = fig.get_figheight() * BARCODE['size']
        ylim = [-1 * yscale,yscale]
        for i,l in enumerate(locf):
            if l:
                axb.plot([x[i],x[i]],ylim,color=BARCODE['color'])
        axb.yaxis.set_visible(False)
        # labels
        if len(xlabel) > 0:
            axb.set_xlabel(xlabel)
        else:
            axb.set_xlabel("rank in ordered feature list")
        if len(ylabel) > 0:
            ax.set_ylabel(ylabel)
        else:
            ax.set_ylabel("enrichment score")
        if len(fileout) > 0:
            plt.savefig(fileout,bbox_inches="tight",dpi=dpi)
        plt.tight_layout()
        plt.show()


    def plot_running_heatmap(self,data,es,loc,focus=None,fileout="",dpi=100,xlabel="",ylabel="",title="",
                        color="forestgreen",alpha:float=0.2,fontsize=14,figsize=(6,6),
                        barcode_params=None,
                        heatmap_params=None):
        """
        visualize a result of enrichment analysis
        
        Parameters
        ----------
        data: array
            raw data to be visualized as a heatmap

        es: 2d array
            enrichment scores of each location (feature x tag/sample)

        loc: 2d array
            location of the indicated tag members in the specimen data

        focus: int
            indicate the No. of the column of interest
            
        """
        BARCODE = {'color':'darkslategray','size':0.15} # hard coding
        HEATMAP = {'cmap':'PRGn','size':0.15,'alpha':0.7} # hard coding
        if barcode_params is not None:
            for k,v in barcode_params.items():
                BARCODE[k] = v
        if heatmap_params is not None:
            for k,v in heatmap_params.items():
                HEATMAP[k] = v
        d = np.sort(data)[::-1]
        y = es.T[focus]
        x = np.arange(1,len(y) + 1,1)
        xmin = np.min(x)
        xmax = np.max(x)
        if figsize!=():
            fig = plt.figure(figsize=figsize)
        else:
            fig = plt.figure()
        spec = gridspec.GridSpec(ncols=1,nrows=3,
                                height_ratios=[1,HEATMAP['size'],BARCODE['size']])
        # prep axes
        if len(title) > 0:
            plt.suptitle(title,fontsize=fontsize)
        plt.rcParams['font.size'] = fontsize
        ax = fig.add_subplot(spec[0,0])
        axh = fig.add_subplot(spec[1,0],sharex=ax)
        axb = fig.add_subplot(spec[2,0],sharex=ax)
        # plot running
        ax = del_spines(ax,['top','right','bottom'])
        ax.plot(x,y,color=color)
        ax.fill_between(x,0,y,facecolor=color,alpha=alpha)
        ax.set_xlim(xmin,xmax)
        ax.xaxis.set_visible(False)
        # plot heatmap
        axh.set_axis_off()
        axh.imshow(d.reshape(1,-1),cmap=HEATMAP['cmap'],aspect='auto',alpha=HEATMAP['alpha'])
        # plot barcode
        axb = del_spines(axb,['top','right','bottom','left'])
        locf = loc[focus]
        yscale = fig.get_figheight() * BARCODE['size']
        ylim = [-1 * yscale,yscale]
        for i,l in enumerate(locf):
            if l:
                axb.plot([x[i],x[i]],ylim,color=BARCODE['color'])
        axb.yaxis.set_visible(False)
        # labels
        if len(xlabel) > 0:
            axb.set_xlabel(xlabel)
        else:
            axb.set_xlabel("rank in ordered feature list")
        if len(ylabel) > 0:
            ax.set_ylabel(ylabel)
        else:
            ax.set_ylabel("enrichment score")
        if len(fileout) > 0:
            plt.savefig(fileout,bbox_inches="tight",dpi=dpi)
        plt.tight_layout()
        plt.show()


# concrete class
class PlotSsGSEA(Plot):
    def plot(self,data,keyword=[],focus=None,order=[],fileout="",dpi=100,xlabel="",
             ylabel="",title="",color="goldenrod",
             palette=["lightyellow","mediumseagreen"],size=8,fontsize=12,figsize=()):
        """
        visualize a result of enrichment analysis
        
        Parameters
        ----------
        es: 2d array
            enrichment scores of each location (feature x tag/sample)
            
        focus: int
            indicate the No. of the column of interest
            
        """
        data = self._preprocess(data,keyword)
        if len(order)==0:
            order = keyword
        if figsize!=():
            fig = plt.figure(figsize=figsize)
        else:
            fig = plt.figure()
        ax = plt.subplot(1,1,1)
        sns.set(style="ticks",palette="pastel")
        if data.shape[0] < 30:
            sns.boxplot(x="indicator",y="value",palette=palette,data=data,order=order,
                        meanline=True,showmeans=True,
                        boxprops={"facecolor":"whitesmoke","color":"whitesmoke"},
                        meanprops={"color":"grey","linestyle":"solid"},
                        medianprops={"color":"whitesmoke"},
                        whiskerprops={"color":"whitesmoke"},
                        capprops={"color":"whitesmoke"},
                        flierprops={"markeredgecolor":"whitesmoke"})
            sns.swarmplot(x="indicator",y="value",color=color,data=data,
                          size=size,order=order)
        else:
            sns.boxplot(x="indicator",y="value",palette=palette,
                        data=data,order=order)
        sns.despine(offset=10,trim=True)
        if len(xlabel) > 0:
            ax.set_xlabel(xlabel,fontsize=fontsize)
        else:
            ax.set_xlabel("")
        if len(ylabel) > 0:
            ax.set_ylabel(ylabel,fontsize=fontsize)
        else:
            ax.set_ylabel("enrichment score",fontsize=fontsize)
        if len(title) > 0:
            ax.set_title(title,fontsize=fontsize)
        ax.tick_params(labelsize=fontsize)
        if len(fileout) > 0:
            plt.savefig(fileout,bbox_inches="tight",dpi=dpi)
        plt.tight_layout()
        plt.show()

    
    def _preprocess(self,data0,keyword=[]):
        """ preprocessing for ssGSEA plots """
        data = data0.copy()
        data = pd.DataFrame(data)
        data.columns = ["value"]
        sample_name = list(data.index)
        if len(keyword)==0:
            raise ValueError("ERROR: give keywords!!")
        elif len(keyword)==1:
            col_ind = ["control" if keyword[0] not in v else keyword[0] for v in sample_name]
        else:
            col_ind = []
            ap = col_ind.append
            for v in sample_name:
                flag = 0
                for w in keyword:
                    if w in v:
                        ap(w)
                        flag += 1
                        break
                if flag==0:
                    ap("notfound")
        data["indicator"] = col_ind
        data = data[data["indicator"]!="notfound"]
        return data


def del_spines(ax,direction=['top','right']):
    """ delete spines """
    for d in direction:
        ax.spines[d].set_visible(False)
    return ax


# -*- coding: utf-8 -*-
"""
Created on Mon Sep 23 13:05:34 2019

PreProcessor class

Analyzer o-- Processor

@author: tadahaya
"""
import pandas as pd
import numpy as np

class Processor():
    def __init__(self):
        pass

    def __vec2tpl(self,mtx,fold=2.0,method="iqr",nmin=None,nmax=None):
        """ convert dataframe into tuple of tags """
        sample_name = list(mtx.columns)
        n_sample = len(sample_name)
        n_feature = len(mtx.index)
        if nmin is None:
            nmin = 15
        if nmax is None:
            nmax = int(0.01*n_feature)
        if method=="std":
            upper,lower = outlier_std(mtx=mtx,fold=fold,axis=0)
        else:
            upper,lower = outlier_iqr(mtx=mtx,fold=fold,axis=0)
        res = []
        ap = res.append
        for i in range(n_sample):
            temp = mtx.iloc[:,i].sort_values(ascending=False)
            up_val = upper[i]
            low_val = lower[i]
            temp_l = list(temp.index)
            upper_tag = set(temp[temp > up_val].index)
            lower_tag = set(temp[temp < low_val].index)
            n_up = len(upper_tag)
            n_low = len(lower_tag)
            if n_up > nmax:
                upper_tag = set(temp_l[:nmax])
            elif n_up < nmin:
                upper_tag = set(temp_l[:nmin])
            if n_low > nmax:
                lower_tag = set(temp_l[-nmax:])
            elif n_low < nmin:
                lower_tag = set(temp_l[-nmin:])
            ap((upper_tag,lower_tag))
        return res


    def vec2set(self,mtx,fold=2.0,two_sided=True,method="iqr",nmin=None,nmax=None):
        """
        convert dataframe into dict of tags

        Parameters
        ----------
        mtx: dataframe
            feature x sample matrix

        fold: float
            determine threshold of outliers

        two_sided: boolean
            determine whether up/down is kept

        method: str
            "std" or "iqr"

        """
        temp = self.__vec2tpl(mtx,fold,method,nmin,nmax)
        if two_sided:
            dic = dict(zip(list(mtx.columns),temp))
        else:
            temp = [v[0] | v[1] for v in temp]
            dic = dict(zip(list(mtx.columns),temp))
        return dic


def outlier_std(mtx,fold,axis=0):
    """ calculate upper and lower values for outlier detection """
    loc = np.mean(mtx.values,axis=axis)
    scale = np.std(mtx.values,axis=axis)        
    upper = loc + fold*scale
    lower = loc - fold*scale
    return upper,lower    
    
    
def outlier_iqr(mtx,fold,axis=0):
    """ calculate upper and lower values for outlier detection """
    q3 = np.percentile(mtx.values,75,axis=axis)
    q1 = np.percentile(mtx.values,25,axis=axis) 
    scale = q3 - q1        
    upper = q3 + fold*scale
    lower = q1 - fold*scale
    return upper,lower


# -*- coding: utf-8 -*-
"""
Created on Mon Sep 23 13:05:34 2019

Adjuster class

Analyzer o-- DataControl o-- Data o-- Adjuster

@author: tadahaya
"""
import pandas as pd

__all__ = ["SeqAdjuster","SetAdjuster","SetTSAdjuster","VectorAdjuster"]

# delegation
class SeqAdjuster():
    def __init__(self):
        pass

    def adjust(self,data,whole,nmin=3):
        """
        adjust seq (set or list) according to whole

        Parameters
        ----------
        data: set or list
            a set or list indicating feature sets

        nmin: int
            indicate minimum number of each set

        """
        return set(data) & whole


class SetAdjuster():
    def __init__(self):
        pass

    def adjust(self,data,whole,nmin=3):
        """
        adjust set (dict) according to whole

        Parameters
        ----------
        data: dict
            dict indicating feature sets

        nmin: int
            indicate minimum number of each set

        """
        new_keys = []
        new_values = []
        ap = new_keys.append
        ap2 = new_values.append
        for k,v in data.items():
            new = v & whole
            if len(new) >= nmin:
                ap(k)
                ap2(new)
        return dict(zip(new_keys,new_values))



class SetTSAdjuster():
    def __init__(self):
        pass

    def adjust(self,data,whole,nmin=3):
        """
        adjust two-sided set (dict) according to whole

        Parameters
        ----------
        data: dict
            dict indicating feature sets

        nmin: int
            indicate minimum number of each set

        """
        new_keys = []
        new_values = []
        ap = new_keys.append
        ap2 = new_values.append
        for k,v in data.items():
            new1 = v[0] & whole
            new2 = v[1] & whole
            if (len(new1) >= nmin) and (len(new2) >= nmin):
                ap(k)
                ap2((new1,new2))
        return dict(zip(new_keys,new_values))


class VectorAdjuster():
    def __init__(self):
        pass

    def adjust(self,data,whole):
        """
        adjust vector (dataframe) according to whole

        Parameters
        ----------
        data: dict
            dict indicating feature sets

        """
        new = set(data.index) & whole
        return data.loc[new,:].sort_index()


# -*- coding: utf-8 -*-
"""
Created on Sat May 30 13:05:34 2020

Data Control class

Analyzer o-- DataControl

@author: tadahaya
"""
import pandas as pd
import numpy as np

from .data import *

# abstract factory
class DataControl():
    def __init__(self):
        self.obj = Data()
        self.ref = Data()
        self.whole = set()

    def set_whole(self,whole):
        """ set whole features """
        self.whole = whole
        self.obj.set_whole(self.whole)
        self.ref.set_whole(self.whole)

    def set_obj(self,data):
        """ create object Data instance for analysis """
        self.obj.set_data(data)

    def set_ref(self,data):
        """ create reference Data instance for analysis """
        self.ref.set_data(data)

    def get_obj(self):
        """ get object data instance """
        return self.obj.get_data()

    def get_ref(self):
        """ get reference data instance """
        return self.ref.get_data()

    def get_whole(self):
        """ get whole """
        return self.whole

    def adjust_obj(self):
        """ adjust object """
        self.obj.adjust()

    def adjust_ref(self):
        """ adjust reference """
        self.ref.adjust()


# concrete class
class FETDataControl(DataControl):
    def __init__(self):
        super().__init__()
        self.obj = SeqData()
        self.ref = SetData()


# concrete class
class BTDataControl(DataControl):
    def __init__(self):
        super().__init__()
        self.obj = SeqData()
        self.ref = SetData()


# concrete class
class GSEADataControl(DataControl):
    def __init__(self):
        super().__init__()
        self.obj = VectorData()
        self.ref = SetData()


# concrete class
class ssGSEADataControl(DataControl):
    def __init__(self):
        super().__init__()
        self.obj = VectorData()
        self.ref = SetData()


# concrete class
class ConnectivityDataControl(DataControl):
    def __init__(self):
        super().__init__()
        self.obj = VectorData()
        self.ref = SetTSData()


# -*- coding: utf-8 -*-
"""
Created on Mon Sep 23 13:05:34 2019

BT class

@author: tadahaya
"""
import pandas as pd
import numpy as np
from itertools import chain
import random
import string

from .process.processor import Processor
from .analyzer import Analyzer
from .data.data_control import BTDataControl
from .calculator._binom import Calculator
from .plot._plot import PlotFET

# concrete class
class BT(Analyzer):
    def __init__(self):
        self.data = BTDataControl()
        self.__process = Processor()
        self.__calc = Calculator()
        self.__plot = PlotFET()
        self.__whole = set()
        self.__ref = dict()
        self.res = pd.DataFrame()


    ### data processing ###
    def check_ref(self,keyword:str):
        """ check contents of reference data """
        if len(self.__ref)==0:
            raise ValueError("!! fit() before this process !!")
        try:
            temp = self.__ref[keyword]
            print("{0}: {1}".format(keyword,temp))
        except KeyError:
            print("!! Wrong keyword !!")
            hit = {v for v in self.__ref.keys() if keyword in v}
            print("perhaps: {}".format(hit))

    def vector2set(self,data,fold:float=3.0,
                   nmin:int=None,nmax:int=None,**kwargs):
        """
        convert dataframe to the outlier set for reference or object

        Parameters
        ----------
        data: dataframe
            feature x sample dataframe

        fold: float
            indicates fold change determining the outliers

        nmin,nmax: int
            indicate the minimum/maximum number of each set 

        """
        return self.__process.vec2set(mtx=data,fold=fold,nmin=nmin,nmax=nmax,
                                      two_sided=False,**kwargs)


    ### data control ###
    def fit(self,data:dict,keep_whole:bool=False,nmin=None):
        """
        set a reference data instance
        
        Parameters
        ----------
        data: dict
            a dictionary of sets like {"XXXX":{"aa","bb"},"YYYY":{"cc","dd","ee"},...}

        keep_whole: boolean
            whether whole features is conserved when already registered

        nmin: int
            indicates the number of features necessary for each set

        """
        self.data.set_ref(data=data)
        self.__ref = self.data.get_ref()
        if keep_whole:
            if len(self.__whole)==0:
                raise ValueError("!! set_whole() or turn off keep_whole !!")
            else:
                self.data.adjust_ref()
        else:
            self.__whole = set(chain.from_iterable(self.__ref.values()))
            self.data.set_whole(self.__whole)
        if nmin is not None:
            temp = self.__ref.copy()
            for k,v in self.__ref.items():
                if len(v) < nmin:
                    del temp[k]
            self.__ref = temp
            self.data.set_ref(data=self.__ref)


    def set_whole(self,whole:set):
        """
        set whole features
        
        Parameters
        ----------
        whole: set
            indicates whole features
        
        """
        self.data.set_whole(whole)
        self.__whole = self.data.get_whole()
        if len(self.data.get_ref())!=0:
            self.data.adjust_ref()

    def get_ref(self):
        """ get reference data instance """
        return self.__ref

    def get_whole(self):
        return self.__whole


    ### calculation ###
    def calc(self,data:set,**kwargs): # realization
        """
        conduct Binomial test and obtain p value corrected for multiple tests
        all elements should be given as ID, except for term
        
        Parameters
        ----------
        data: set
            features of interest to be anlyzed

        correction: str
            indicate method for correcting multiple tests
            depend on "statsmodels.stats.multitest.multipletests"
            
        focus: int
            export results by XX th lowest p value

        mode: str
            indicate the type of significant judging
            "greater", "two-sided", or "less"
    
        """
        self.data.set_obj(data)
        self.res = self.__calc.calc(obj=self.data.get_obj(),ref=self.__ref,whole=self.__whole,**kwargs)
        return self.res


    ### visualization ###
    def set_res(self,df):
        """ set a result """
        self.res = df


    def plot(self,display_num:int=5,**kwargs): # realization
        """
        visualize a result of enrichment analysis

        Parameters
        ----------
        display_num: int
            determine how many groups are visualized

        fileout: str
            indicate the path for the output image

        dpi: int
            indicate dpi of the output image
            
        thresh: float
            indicate the threshold value of adjusted p value to be colored

        xlabel,ylabel: str
            indicate the name of x and y axes

        title: str
            indicate the title of the plot

        color: str
            indicate the color of the bars

        alpha: float
            indicate transparency of the bars: (0,1)

        height: float
            indicate the height of the bars

        fontsize: float
            indicate the fontsize in the plot

        textsize: float
            indicate the fontsize of the texts in the bars

        figsize: tuple
            indicate the size of the plot

        """
        self.__plot.plot(res=self.res,focus=display_num,**kwargs)


    ### other ###
    def generate_test_data(self):
        """ generate data for test """    
        # prepare reference
        key = [''.join(random.choices(string.ascii_letters,k=5)) for i in range(10)]

        def fxn():
            return {random.randint(0,50) for j in range(random.randint(5,30))}
        
        val = [fxn() for i in range(9)] + [{1,2,3,4,5,6,7}]
        ref = dict(zip(key,val))

        # prepare object
        obj = {1,2,3,4,5,6,7,8,9,100,1000}
        print("generate test data as follows:")
        print('    ref: dict, {"XXXX":{"aa","bb"},"YYYY":{"cc","dd","ee"},...}')
        print('    obj: set, {"aa","bb","cc"}')
        return ref,obj



# -*- coding: utf-8 -*-
"""
Created on Mon Sep 23 13:05:34 2019

GSEA class

@author: tadahaya
"""
import pandas as pd
import numpy as np
from itertools import chain
import random
import string

from .process.processor import Processor
from .analyzer import Analyzer
from .data.data_control import GSEADataControl
from .calculator._gsea import Calculator
from .plot._plot import PlotGSEA

# concrete class
class GSEA(Analyzer):
    def __init__(self):
        self.data = GSEADataControl()
        self.__process = Processor()
        self.__calc = Calculator()
        self.__plot = PlotGSEA()
        self.__whole = set()
        self.__obj = pd.DataFrame()
        self.__ref = dict()
        self.__method = ""
        self.alpha = 0.0
        self.res = pd.DataFrame()


    ### data processing ###
    def check_ref(self,keyword:str):
        """ check contents of reference data """
        if len(self.__ref)==0:
            raise ValueError("!! fit() before this process !!")
        try:
            temp = self.__ref[keyword]
            print("{0}: {1}".format(keyword,temp))
        except KeyError:
            print("!! Wrong keyword !!")
            hit = {v for v in self.__ref.keys() if keyword in v}
            print("perhaps: {}".format(hit))

    def vector2set(self,data,fold:float=3.0,
                   nmin:int=None,nmax:int=None,**kwargs):
        """
        convert dataframe to the outlier set for reference

        Parameters
        ----------
        data: dataframe
            feature x sample dataframe

        fold: float
            indicates fold change determining the outliers

        nmin,nmax: int
            indicate the minimum/maximum number of each set 

        """
        return self.__process.vec2set(mtx=data,fold=fold,nmin=nmin,nmax=nmax,
                                      two_sided=False,**kwargs)


    ### data control ###
    def fit(self,data:dict,keep_whole:bool=False,nmin=None):
        """
        set a reference data instance
        
        Parameters
        ----------
        data: dict
            a dictionary of sets like {"XXXX":{"aa","bb"},"YYYY":{"cc","dd","ee"},...}

        keep_whole: boolean
            whether whole features is conserved when already registered

        nmin: int
            indicates the number of features necessary for each set

        """
        self.data.set_ref(data=data)
        self.__ref = self.data.get_ref()
        if keep_whole:
            if len(self.__whole)==0:
                raise ValueError("!! set_whole() or turn off keep_whole !!")
            else:
                self.data.adjust_ref()
        else:
            self.__whole = set(chain.from_iterable(self.__ref.values()))
            self.data.set_whole(self.__whole)
        if nmin is not None:
            temp = self.__ref.copy()
            for k,v in self.__ref.items():
                if len(v) < nmin:
                    del temp[k]
            self.__ref = temp
            self.data.set_ref(data=self.__ref)

    def set_whole(self,whole:set):
        """
        set whole features
        
        Parameters
        ----------
        whole: set
            indicates whole features
        
        """
        self.data.set_whole(whole)
        self.__whole = self.data.get_whole()
        if len(self.data.get_ref())!=0:
            self.data.adjust_ref()

    def get_ref(self):
        """ get reference data instance """
        return self.data.get_ref()

    def get_obj(self):
        """ get an object data instance """
        return self.data.get_obj()

    def get_whole(self):
        return self.__whole

    def get_calculator(self):
        """ get calculater object """
        return self.__calc


    ### calculation ###
    def calc(self,data,method:str="standard",alpha:float=0.0): # realization
        """
        conduct GSEA

        Parameters
        -------
        data: dataframe
            feature x sample dataframe

        method: str
            indicate a method for calculating the enrichment score
            "starndard": employed in the original paper Barbie, et al, 2009
            "kuiper": Kuiper test statistics, good when up/down genes are mixed, tail sensitive
            "gsva": GSVA like statistics, good when unidirection (ex. up only)

        alpha: float, (0,1]
            indicate weight of center
            0 means no weight and is employed well

        Returns res
        -------
        res: df
            gene set enrichment score

        """
        self.__method = method
        self.__alpha = alpha
        if method=="standard":
            self.__calc.to_standard()
            print("Standard method")
        elif method=="kuiper":
            self.__calc.to_kuiper()
            print("Kuiper method")
        elif method=="gsva":
            self.__calc.to_gsva()
            print("GSVA method")
        else:
            raise ValueError("!! Wrong method: choose 'standard', 'kuiper', or 'gsva' !!")
        self.data.set_obj(data)
        self.__obj = self.data.get_obj()
        temp = self.__obj.copy()
        col = list(temp.columns)
        res = []
        ap = res.append
        for v in col:
            ap(self.__calc.calc(obj=temp[v],ref=self.__ref,alpha=alpha))
        res = pd.concat(res,axis=1,join="inner")
        res.columns = col
        self.res = res
        return res


    def normalize_score(self):
        """
        normalize enrichment score with maximum value
        Note that this method can be applied
        only when the standard method was employed for calculation
        
        """
        if self.res.empty:
            raise ValueError("!! No result stored: calc() before this method !!")
        n = len(self.__whole)
        key = list(self.__ref.keys())
        val = list(self.__ref.values())
        dic = dict(zip(key,[len(v) for v in val]))
        idx = list(self.res.index)
        t = np.array([dic[v] for v in idx])
        if type(self.res)==type(pd.DataFrame()):
            kmax = np.c_[1 - t/n]
            val = self.res.values/kmax
        else:
            kmax = 1 - t/n
            val = self.res.values.flatten()/kmax
        return pd.DataFrame(val,columns=self.res.columns,index=idx)


    ### visualization ###
    def set_res(self,data):
        """ set a result """
        self.res = data


    def plot(self,sample_name:str=None,highlight:list=[],ylabel:str="enrichment score",**kwargs): # realization
        """
        visualize a result of GSEA
        Parameters
        ----------
        sample_name: str
            indicate the sample name to be visualized

        highlight: list
            indicate the reference keys to be highlightened in the plot

        fileout: str
            indicate the path for the output image

        dpi: int
            indicate dpi of the output image
            
        ylabel: str
            indicate the name of y axis

        title: str
            indicate the title of the plot

        color: str
            indicate the color of the bars
            
        fontsize: float
            indicate the fontsize in the plot

        size: float
            indicate the size of the plot

        figsize: tuple
            indicate the size of the plot
        """
        if sample_name is None:
            col = list(self.res.columns)
            for v in col:
                self.__plot.plot(data=self.res[v],highlight=highlight,ylabel=ylabel,**kwargs)
        else:
            focused = self.res[sample_name]
            self.__plot.plot(data=focused,highlight=highlight,ylabel=ylabel,**kwargs)
        

    def plot_running(
            self,sample_name:str=None,fterm:str="",title:str="",
            barcode_params:dict=None,
            heatmap:bool=True,heatmap_params:dict=None,**kwargs
            ): # realization
        """
        visualize a result of GSEA, running sum plot

        Parameters
        ----------
        sample_name: str
            indicate the sample name to be visualized

        fterm: str or int
            indicate the term of interest or the corresponding No.

        fileout: str
            indicate the path for the output image

        dpi: int
            indicate dpi of the output image
            
        xlabel,ylabel: str
            indicate the name of x and y axes

        title: str
            indicate the title of the plot

        color: str
            indicate the color of the bars

        fontsize: float
            indicate the fontsize in the plot

        figsize: tuple
            indicate the size of the plot

        barcode_params: dict
            indicate the parameters for barcode
            color: str
            size: float

        heatmap_params: dict
            indicate the parameters for heatmap
            cmap: str
            size: float
            alpha: float

        """
        if sample_name is None:
            raise ValueError("!! Indicate sample_name !!")
        data = self.__obj.copy()
        focused = data[sample_name]
        res = self.__calc.calc(obj=focused,ref=self.__ref,alpha=self.__alpha)
        es = self.__calc.es
        keys = self.__calc.keys
        if len(es)==0:
            raise ValueError("!! No Enrichment score !!")
        else:
            if type(fterm)==str:
                focus_key = fterm
                try:
                    focus_num = keys.index(fterm)
                except KeyError:
                    print("!! Wrong key: change fterm !!")
            elif type(fterm)==int:
                if len(es) < fterm:
                    raise ValueError("!! focused number is larger than column No. !!")
                focus_num = fterm
                focus_key = keys[fterm]
            else:
                raise TypeError("!! Wrong type: focus should be str or int !!")
        loc = self.__calc.loc
        if len(title) > 0:
            if heatmap:
                self.__plot.plot_running_heatmap(
                    data=focused.values,es=es,loc=loc,focus=focus_num,title=title,
                    barcode_params=barcode_params,heatmap_params=heatmap_params,**kwargs
                    )
            else:
                self.__plot.plot_running(
                    es=es,loc=loc,focus=focus_num,barcode_params=barcode_params**kwargs
                    )
        else:
            if heatmap:
                self.__plot.plot_running_heatmap(
                    data=focused.values,es=es,loc=loc,focus=focus_num,title=title,
                    barcode_params=barcode_params,heatmap_params=heatmap_params,**kwargs
                    )
            else:
                self.__plot.plot_running(
                    es=es,loc=loc,focus=focus_num,barcode_params=barcode_params,**kwargs
                    )


    ### other ###
    def generate_test_data(self):
        """ generate data for test """    
        # prepare reference
        key = [''.join(random.choices(string.ascii_letters, k=5)) for i in range(10)]

        def fxn():
            return {random.randint(0,50) for j in range(random.randint(10,100))}

        val = [fxn() for i in range(9)] + [{1,2,3,4,5,6,7}]
        ref = dict(zip(key,val))

        # prepare object
        obj = [np.random.randn(10000) for i in range(3)]
        obj = pd.DataFrame({"xxxx":obj[0],"yyyy":obj[1],"zzzz":obj[2]})
        print("generate test data as follows:")
        print('    ref: dict, {"XXXX":{"aa","bb"},"YYYY":{"cc","dd","ee"},...}')
        print('    obj: dataframe, feature x sample')
        return ref,obj



# -*- coding: utf-8 -*-
"""
Created on Mon Sep 23 13:05:34 2019

Data class

Analyzer o-- DataControl *-- Data

@author: tadahaya
"""
import pandas as pd
import numpy as np

from .adjuster import *

__all__ = ["Data","SeqData","SetData","SetTSData","VectorData"]

# abstract class
class Data():
    def __init__(self):
        self.data = None
        self.whole = set()

    def set_data(self,data):
        """ set data """
        raise NotImplementedError

    def set_whole(self,whole):
        """ set whole """
        self.whole = whole

    def get_data(self):
        """ get data """
        return self.data

    def get_whole(self):
        """ get whole """
        return self.whole

    def adjust(self,**kwargs):
        """ adjust data to the indicated whole """
        raise NotImplementedError


# concrete class
class SeqData(Data):
    """ data given as a set """
    def __init__(self):
        super().__init__()
        self.data = {}
        self.__adj = SeqAdjuster()

    def set_data(self,data):
        """ load data """
        if type(data)!=set:
            raise TypeError("!! data should be a set !!")
        self.data = data

    def adjust(self,**kwargs):
        """ adjust data to the indicated whole """
        self.data = self.__adj.adjust(self.data,self.whole,**kwargs)


# concrete class
class SetData(Data):
    """ data given as a dict of {term:sets of group} """
    def __init__(self):
        super().__init__()
        self.data = dict()
        self.__adj = SetAdjuster() # private

    def set_data(self,data):
        """ load data """
        if type(data)!=dict:
            raise TypeError("!! data should be a dict !!")
        self.data = data

    def adjust(self,**kwargs):
        """ adjust data to the indicated whole """
        self.data = self.__adj.adjust(self.data,self.whole,**kwargs)


# concrete class
class SetTSData(Data):
    """ data given as a dict of {term:tuples of up/down tags} """
    def __init__(self):
        super().__init__()
        self.data = dict()
        self.__adj = SetTSAdjuster() # private

    def set_data(self,data):
        """ load data """
        if type(data)!=dict:
            raise TypeError("!! data should be a dict !!")
        elif type(list(data.values())[0])!=tuple:
            raise TypeError("!! data should be a dict of tuples of up/down tags !!")
        self.data = data

    def adjust(self,**kwargs):
        """ adjust data to the indicated whole """
        self.data = self.__adj.adjust(self.data,self.whole,**kwargs)


# concrete class
class VectorData(Data):
    """ data given as a dataframe """
    def __init__(self):
        super().__init__()
        self.data = pd.DataFrame()
        self.__adj = VectorAdjuster() # private

    def set_data(self,data):
        """ load data """
        if type(data)==pd.core.series.Series:
            self.data = pd.DataFrame(data)
        elif type(data)==pd.core.frame.DataFrame:
            self.data = data
        else:
            raise TypeError("!! data should be a dataframe !!")

    def adjust(self,**kwargs):
        """ adjust data to the indicated whole """
        self.data = self.__adj.adjust(self.data,self.whole,**kwargs)


# -*- coding: utf-8 -*-
"""
Created on Mon Sep 23 13:05:34 2019

Analyzer class

@author: tadahaya
"""
import pandas as pd
import numpy as np

from .data.data_control import DataControl
from .process.processor import Processor

# abstract class
class Analyzer():
    def __init__(self):
        self.data = DataControl()
        self.__process = Processor()
        self.__calc = None
        self.__plot = None
        self.__whole = set()
        self.__ref = None
        self.__obj = None
        self.res = None


    ### data processing ###
    def check_ref(self):
        """ check contents of reference data """
        raise NotImplementedError

    def vector2set(self):
        """ convert dataframe to the outlier set """
        raise NotImplementedError


    ### data control ###
    def fit(self,data=None):
        """ set a reference data instance """
        raise NotImplementedError

    def set_whole(self,whole=None):
        """ set whole features """
        raise NotImplementedError

    def get_ref(self):
        """ get reference data instance """
        raise NotImplementedError

    def get_whole(self):
        """ get whole set """
        raise NotImplementedError

    def get_res(self):
        """ get result """
        raise NotImplementedError


    ### calculator ###
    # abstract method
    def calc(self):
        """ conduct calculation """
        raise NotImplementedError


    ### visualization ###
    # abstract method
    def set_res(self):
        """ set a result """
        raise NotImplementedError

    def plot(self):
        """ visualize the result """
        raise NotImplementedError


# -*- coding: utf-8 -*-
"""
Created on Tue Jan 14 15:48:00 2020

Gene set enrichment analysis

@author: tadahaya
"""
import pandas as pd
import numpy as np

### preprocessing ###
class Process():
    """ prepare data to be analyzed """
    def __init__(self):
        self.__process = GSEAProcess()

    def do(self,data,ref,**kwargs):
        return self.__process.do(data,ref,**kwargs)

    def to_gsea(self):
        self.__process = GSEAProcess()

    def to_ssgsea(self):
        self.__process = ssGSEAProcess()

    def to_expssgsea(self):
        self.__process = ExpssGSEAProcess()


class GSEAProcess():
    def do(self,data,ref):
        """
        prepare target data for GSEA
        
        data: series
            a series of interest data

        """
        return data.sort_values(ascending=False),ref


class ssGSEAProcess():
    def do(self,data,ref,fterm):
        """
        prepare target data for ssGSEA focused mode
        
        data: dataframe
            a dataframe of interest data (feature x sample)
        
        fterm: str
            a term of interest in refrence data set
        
        """
        return _sort_data(data),ref[fterm]


class ExpssGSEAProcess():
    def do(self,data,ref):
        """
        prepare target data for ssGSEA exploratory mode
        
        data: series
            a series of interest data

        """
        return data.sort_values(ascending=False),ref


### scoreing method ###
class Method():
    """ indicate scoreing method of enrichment scores for output """
    def __init__(self):
        self.__method = StandardMethod()

    def calc(self,es,names):
        return self.__method.calc(es,names)

    def to_standard(self):
        self.__method = StandardMethod()

    def to_kuiper(self):
        self.__method = KuiperMethod()

    def to_gsva(self):
        self.__method = GSVAMethod()


class StandardMethod():
    def calc(self,es,names):
        """ calculate Enrichment score from es with the standard method """
        res = pd.DataFrame(np.max(np.abs(es),axis=0),index=names,
                            columns=["ES"]).sort_values(by="ES",ascending=False)
        return res


class KuiperMethod():
    def calc(self,es,names):
        """ calculate Enrichment score from es with the method in Kuiper method """
        res = pd.DataFrame(np.abs(np.max(es,axis=0)) + np.abs(np.min(es,axis=0)),
                            index=names,columns=["ES"]).sort_values(by="ES",ascending=False)
        return res


class GSVAMethod():
    def calc(self,es,names):
        """ calculate Enrichment score from es with the method in GSVA """
        maxi = np.max(es,axis=0)
        mini = np.min(es,axis=0)
        res = pd.DataFrame(np.where(maxi < 0,0,maxi) - np.where(mini > 0,0,mini),
                            index=names,columns=["ES"]).sort_values(by="ES",ascending=False)
        return res


### Algorithm ###
class Algorithm():
    """ indicate analysis algorithm """
    def __init__(self,):
        self.__algorithm = GSEAAlgorithm()

    def calc(self,data,tag,alpha):
        """
        Returns: (es,keys,loc)
        ----------
        es: 2d array
            enrichment scores of each location (feature x tag/sample)

        keys: list
            tag names or sample names for output

        loc: 2d array
            location of tag-positive elements in data (tag/sample x feature)

        """
        return self.__algorithm.calc(data,tag,alpha)

    def to_gsea(self):
        self.__algorithm = GSEAAlgorithm()

    def to_ssgsea(self):
        self.__algorithm = ssGSEAAlgorithm()

    def to_expssgsea(self):
        self.__algorithm = ExpssGSEAAlgorithm()


class GSEAAlgorithm():
    def calc(self,data,tag,alpha=0):
        """
        standard GSEA algorithm
        
        Parameters
        ----------
        data: series
            a sorted series of interest data in descending order (high values get high ranks)
            
        tag: dict
            keys: group name
            values: members of each group

        """
        keys = list(tag.keys())
        values = list(tag.values())
        sorted_list = list(data.index)
        sorted_val = np.abs(list(data))
        n = len(sorted_list)
        ### grasp where the tagged genes
        loc = _location(sorted_list,values)
        ### calculate posi
        posi0 = np.power(sorted_val,alpha)*loc
        den_p = np.sum(posi0,axis=1)
        posi = _accumulative(posi0,axis=1)/den_p
        ### calculate nega
        nega0 = 1 - loc
        # n_nh = n - np.array([len(v) for v in values])
        n_nh = np.sum(nega0,axis=1) # 211014
        nega = _accumulative(nega0,axis=1)/n_nh
        ### calculate ES
        es = posi - nega
        return es,keys,loc


class ssGSEAAlgorithm():
    def calc(self,data,tag,alpha=0.25):
        """
        ssGSEA focusing on a tag
        
        Parameters
        ----------
        data: dataframe
            a dataframe composed of sorted features (descending; high values get high ranks)
            each column corresponds to each sample
            
        tag: set
            a set of elements constituting a signature        
        
        """
        keys = list(data.columns)
        n = data.shape[0]
        ### grasp where the tagged genes
        tag = list(tag)
        loc = data.replace(tag,1).T
        loc = (loc.values==1)
        ### calculate posi
        full = np.arange(1,n + 1,1)[::-1]
        posi0 = np.power(full,alpha)*loc
        den_p = np.sum(posi0,axis=1)
        posi = _accumulative(posi0,axis=1)/den_p
        ### calculate nega
        nega0 = 1 - loc
        n_nh = np.sum(nega0,axis=1)
        nega = _accumulative(nega0,axis=1)/n_nh
        ### calculate ES
        es = posi - nega
        return es,keys,loc


class ExpssGSEAAlgorithm():
    def calc(self,data,tag,alpha=0.25):
        """
        ssGSEA for exploratory analysis by searching whole tags
        
        Parameters
        ----------
        data: series
            a sorted series of interest data in descending order (high values get high ranks)
            
        tag: dict
            keys: group name
            values: members of each group
        
        """        
        keys = list(tag.keys())
        values = list(tag.values())
        data = list(data.index)
        n = len(data)
        ### grasp where the tagged genes
        loc = _location(data,values)
        ### calculate posi
        full = np.arange(1,n + 1,1)[::-1]
        posi0 = np.power(full,alpha)*loc
        den_p = np.sum(posi0,axis=1)
        posi = _accumulative(posi0,axis=1)/den_p
        ### calculate nega
        nega0 = 1 - loc
        n_nh = np.sum(nega0,axis=1)
        nega = _accumulative(nega0,axis=1)/n_nh
        ### calculate ES
        es = posi - nega
        return es,keys,loc


### Context ###
class Calculator():
    """ GSEA client """
    def __init__(self):
        self.__process = Process()
        self.__algorithm = Algorithm()
        self.__method = Method()
        self.es = np.array([[],[]]) # for plot
        self.loc = np.array([[],[]]) # for plot
        self.keys = []
        self.res = pd.DataFrame()


    def calc(self,obj,ref,alpha,**kwargs):
        """
        conduct GSEA

        Parameters
        ----------
        alpha: float, (0,1]
            indicate weight of center
            0.25 is derived from the original paper of ssGSEA (Barbie,et al.,2009)
        
        fterm: str
            a term of interest in refrence data set

        """
        obj,tag = self.__process.do(obj,ref,**kwargs)
        self.es,self.keys,self.loc = self.__algorithm.calc(obj,tag,alpha)
        self.res = self.__method.calc(self.es,self.keys)
        return self.res


    def get_details(self):
        """
        obtain detailed data for visualization

        Returns (es,loc,keys,res)
        ----------
        es: 2d array
            enrichment scores of each location (feature x tag/sample)

        loc: 2d array
            location of tag-positive elements in data (tag/sample x feature)

        keys: list
            tag names or sample names for output

        """
        return self.es,self.loc,self.keys,self.res


    def to_gsea(self):
        self.__process.to_gsea()
        self.__algorithm.to_gsea()

    def to_ssgsea(self):
        self.__process.to_ssgsea()
        self.__algorithm.to_ssgsea()

    def to_expssgsea(self):
        self.__process.to_expssgsea()
        self.__algorithm.to_expssgsea()

    def to_standard(self):
        self.__method.to_standard()

    def to_kuiper(self):
        self.__method.to_kuiper()

    def to_gsva(self):
        self.__method.to_gsva()


### Common Functions ###
def _location(data,tags):
    """
    determine the location of the interest elements in a sample
    returns tag No. x feature No. matrix
    
    Parameters
    ----------
    data: list
        sorted list by values (descending)

    tags: list
        a list of tag sets

    """
    res = []
    ap = res.append
    for tag in tags:
         ap([(ele in tag) for ele in data])
    return np.array(res)


def _accumulative(X,axis=0):
    """
    calculate accumulative sum
    
    Parameters
    ----------
    axis: 0 or 1
        determine sum direction
    
    """
    if axis==1:
        X = X.T
    x = np.zeros(X.shape[1])
    res = []
    ap = res.append
    for v in X:
        x += v
        ap(x.copy())
    res = np.array(res)
    return res


def _sort_data(df,ascending=False):
    """
    sort dataframe by values in columns and return corresponding indices
    
    Parameters
    ----------
    df: dataframe

    ascending: boolean
        indicates sort direction

    """
    col = list(df.columns)
    res = list(map(lambda x: list(df[x].sort_values(ascending=ascending).index),col))
    return pd.DataFrame(res,index=col).T



# -*- coding: utf-8 -*-
"""
Created on Fri Jul 10 14:30:27 2020

@author: tadahaya
"""
from setuptools import setup,find_packages
from setuptools.command.test import test as TestCommand
import sys

__version__ = "1.0.4" # major.minor[.patch[.sub]] 

sys.path.append('.\\main')
sys.path.append('.\\tests')

REQUIRED_PKG = ['numpy','scipy','pandas','statsmodels','tqdm','xlrd','matplotlib','seaborn']

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='enan',
    version=__version__,
    description="a package for ENrichment ANalyses", # short description
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/mizuno-group',
    author='tadahaya mizuno',
    author_email='tadahaya@gmail.com',
    license='MIT',
    classifiers=['Development Status :: 5 - Production/Stable'
                 ,'Environment :: Win32 (MS Windows)'
                 ,'Framework :: IPython'
                 ,'Intended Audience :: Science/Research'
                 ,'Operating System :: Microsoft :: Windows :: Windows 10'
                 ,'Programming Language :: Python :: 3'
                 ,'Topic :: Scientific/Engineering :: Bio-Informatics'],
    keywords=['omics','bioinformatics','transcriptome','chemoinformatics'],
    install_requires=REQUIRED_PKG,
    python_requires='>=3.6',
    include_package_data=True,
    package_data={'enapy':['*.txt','*.ignore','*.ipynb','*.md']},
    packages=find_packages(exclude=('tests', 'docs')),
    zip_safe=False
    )


# -*- coding: utf-8 -*-
"""
Created on Mon Sep 23 12:59:35 2019

Binomial test

@author: tadahaya
"""
import pandas as pd
import numpy as np
import scipy.stats as stats
import statsmodels.stats.multitest as multitest
from scipy.stats import rankdata


class Calculator():
    def __init__(self):
        self.res = pd.DataFrame()


    def calc(self,obj,ref,whole,focus=None,**kwargs):
        self.res = do_binom(obj,ref,whole,focus=None,**kwargs)
        return self.res


    def get_details(self):
        return self.res


def do_binom(obj,ref,whole,focus=None,correction="fdr_bh",mode="greater"):
    """
    conduct Binomial test and obtain p value corrected for multiple tests
    all elements should be given as ID, except for term
    
    Parameters
    ----------
    obj: set
        a set of variables in signature of interest
        
    ref: dict
        a dict of term and member list of datasets

    whole: set
        a set of whole variables adjusted between datasets and interest 
        
    method: str
        indicate method for correcting multiple tests
        depend on "statsmodels.stats.multitest.multipletests"
        
    focus: int
        export results by XX th lowest p value

    mode: str
        indicate the type of significant judging
        "greater", "two-sided", or "less"

    """
    pval = []
    overlap = []
    total = []
    hit = []
    ap = pval.append
    ap2 = overlap.append
    ap3 = total.append
    ap4 = hit.append
    n_whole = len(whole)
    keys = list(ref.keys())
    values = list(ref.values())
    for m in values:
        n_set = len(m)
        pc = n_set/n_whole
        inter = obj & m
        n_inter = len(inter)
        ap(stats.binom_test(n_inter,n_set,pc,alternative=mode))
        ap2(inter)
        ap3(len(m))
        ap4(n_inter)
    if len(pval)==0:
        res = pd.DataFrame(columns=["p value","adjusted p value","overlap","hit No.",
                                    "total No."])
    else:
        res = pd.DataFrame({"p value":pval,"overlap":overlap,
                            "hit No.":hit,"total No.":total},index=keys).sort_values(by="p value")
        fxn = lambda x: len(x) > 0
        res = res[res["overlap"].map(fxn)]
        if res.shape[0]!=0:
            res["adjusted p value"] = multitest.multipletests(res["p value"],alpha=0.05,method=correction)[1]
            res = res.sort_values(by="p value")
            res = res.loc[:,["p value","adjusted p value","overlap","hit No.","total No."]] # sort
    if (focus is None) or (res.shape[0]==0):
        pass
    else:
        res = res.iloc[:focus,:]
    return res


# -*- coding: utf-8 -*-
"""
Created on Wed Dec 25 15:46:32 2019

@author: tadahaya
"""

from .binom import BT
from .connect import Connect
from .fet import FET
from .gsea import GSEA
from .ssgsea import ssGSEA

__copyright__    = 'Copyright (C) 2021 MIZUNO Tadahaya'
__version__      = '1.0.4'
__license__      = 'MIT'
__author__       = 'MIZUNO Tadahaya'
__author_email__ = 'tadahaya@gmail.com'